% This function is part of the reproducible research addendum of the research
% paper:
% T. Necciari, N. Holighaus, P. Balazs, Z. Prusa, P. Majdak, and O. Derrien
% "Audlet Filter Banks: A Versatile Analysis/Synthesis Framework using Auditory Frequency Scales"
%
% This file along with the auxiliary files
% - mixtured.wav
% - femvocd1.wav
% - ronin55d1.wav
% - gen_bmask_voice.m
% reproduces the presented audio separation experiment.
% 
% Four FB systems are evaluated:
%   gfbSOA: Compactly-supported gammatone system with a suboptimal synthesis stage 
%       (time-reversed analysis filters) 
%   gfbOPT: Compactly-supported gammatone system with an optimal synthesis
%       stage (approximation using CG)
%   hannOPT: Hann system with an optimal synthesis stage 
%   STFT: Gabor transform using a 1024-point Hann window. Synthesis is
%       achieved by the dual window.
%
% To run this file, you will need the LTFAT Toolbox V.2.2.1 or later,
% available at: http://ltfat.github.io
% Furthermore, you will need the PEASS Toolbox V.2.0, available at: http://bass-db.gforge.inria.fr/peass/
% 
% Run without input parameters, the experiment presented in the manuscript
% will be reproduced, that is, for \beta = [1 1/6] and R_t = [1.1 1.5 4].
% The values of \beta and R_t can be adjusted using the corresponding parameters.
% Note that values of R_t smaller than 1 might lead to instability.
% 
% The second input parameter 'PLOT' can be set to 0 to suppress plotting of
% the results.  
% 
% Author: T.Necciari and N. Holighaus

function Qmeasures = hann_gtfb_separation_exp2(Rt,beta,PLOT)

if nargin < 3
    PLOT = 0; % Default: plot exemplary results
    if nargin < 2 
        Rt = [1.1 1.5 4]; % Low, medium and high redundancy
        if nargin < 1
            beta = [1 1/6]; % coarse and fine tuning
        end
    end
end

%% Load test signal and set up the filterbank parameters

[s,sr]=wavload('signals/mixtured');
L = length(s);
Ls = ceil(L/sr)*sr;
s = [s;zeros(Ls-L,1)];

% Set filters' spacing and bandwidth
fmin = 20;
fmax = 10000;
V = 6;
spac = 1/V;
B = length(beta);
A = length(Rt);

% Prepare table of results
Qmeasures = zeros(3*A,3*B);
VOC_ORG = wavload('signals/femvocd1.wav'); % Original voice source
AUD_ORG = wavload('signals/ronin55d1.wav'); % Original music source

for ii=1:B
    
    %% Generate the masks for the current value of \beta
    [MASK_Hann,MASK_GT,MASKGAB] = gen_bmask_voice(beta(ii),-25,fmin,fmax,V);
%     The threshold value of -25 is appropriate. The threshold affects the separation performance.
    MASK_Hann = double(MASK_Hann > 0);
    MASK_GT = double(MASK_GT > 0);
    MASKGAB = double(MASKGAB > 0);

    % Compute the redundancy factor to apply in order to achieve both systems
    % with 'true_red'
    for jj=1:A
        fprintf('Currently at beta = %g and target redundancy Rt = %g, generating filters and masks\n',beta(ii),Rt(jj));
        
        [erblets,a_H,fc,~] = audfilters(sr,Ls,'fractional','spacing',spac,'bwmul',beta(ii),'fmin',fmin,'fmax',fmax,'redtar',Rt(jj));
        [gfb,a_gt,~,~] = audfilters(sr,Ls,'fractional','gammatone','spacing',spac,'bwmul',beta(ii),'fmin',fmin,'fmax',fmax,'redtar',Rt(jj));
        
        % Comparison with Gabor transform
        M = 1024;%2048
        a_gab = round(M/Rt(jj));
        c_dgt = dgtreal(s,{'hann',M},a_gab,M);
%         Test display
%         plotdgtreal(c_dgt,a_gab,M,sr,60);
        
        % These are the final downsampling factors
        DF_hann = a_H(:,1)./a_H(:,2);
        DF_gfb = a_gt(:,1)./a_gt(:,2);

        % Compute and display redundancies for verification
        R_hann = sum(2./DF_hann)-sum(1./DF_hann([1,end]));
        R_gfb = sum(2./DF_gfb)-sum(1./DF_gfb([1,end]));
        R_dgt = 2*numel(c_dgt)/Ls;
        fprintf('Actual redundancy of the system with Hann filters: %g \n', R_hann);
        fprintf('Actual redundancy of the systems with gammatone filters: %g \n', R_gfb);
        fprintf('Actual redundancy of the Gabor transform: %g \n', R_dgt);

        %% Compute the filterbank analysis
        disp('Processing...')
        c_gfb = filterbank(s,gfb,a_gt);
        c_hann = filterbank(s,erblets,a_H);

        % Setup empty cell arrays for the modified coefficients
        chans = length(c_hann); % Determine the number of channels
        c_hann_masked = cell(1,chans);
        c_gfb_masked = c_hann_masked;
        for kk = 1:chans
            % Interpolate the masks to the channel's length
            MASKkk_gt = interp1(MASK_GT(kk,:),linspace(1,size(MASK_GT,2),length(c_gfb{kk})));
            MASKkk_H = interp1(MASK_Hann(kk,:),linspace(1,size(MASK_Hann,2),length(c_hann{kk})));
            c_hann_masked{kk} = c_hann{kk}.*MASKkk_H'; % Apply the mask to the ERBlet coefficients
            c_gfb_masked{kk} = c_gfb{kk}.*MASKkk_gt'; % Apply the mask to the gammatone coefficients
        end
        % For Gabor transform
        MASK_dgt = interp2(MASKGAB,linspace(1,size(MASKGAB,2),size(c_dgt,2)),linspace(1,size(MASKGAB,1),size(c_dgt,1)).');
        c_dgt_masked = c_dgt.*MASK_dgt;
        
        %% Synthesize the masked signal (separated voice)
        % Gammatone synthesis (state-of-the-art "SOA" system)
        % Time-reversed analysis filters
        gfbSOA_sep_voc = 2*real(ifilterbank(c_gfb_masked,gfb,a_gt));
        gfbSOA_sep_voc = gfbSOA_sep_voc/mean(filterbankresponse(gfb,a_gt,L,'real'));
%         gfb_sep_aud2 = s-gfb_sep_voc2;

        % Gammatone synthesis (optimal system)
        % Non-painless, apply CG
        gfbOPT_sep_voc = ifilterbankiter(c_gfb_masked,gfb,a_gt,'pcg');

        % Hann-Audlet synthesis
        hann_sep_voc = ifilterbankiter(c_hann_masked,erblets,a_H,'pcg');    

%       Gabor synthesis
        dgt_sep_voc = idgtreal(c_dgt_masked,{'dual','hann',M},a_gab,M,L);
        
        disp('Saving results...')
        % Save results as wav files for objective evaluation
        wavsave(hann_sep_voc,sr,['signals/targetEstimate_Hann_Rt', num2str(Rt(jj)),'_beta', num2str(beta(ii)),'.wav']);
        wavsave(gfbOPT_sep_voc,sr,['signals/targetEstimate_gfbOPT_Rt', num2str(Rt(jj)),'_beta', num2str(beta(ii)),'.wav']);
        wavsave(gfbSOA_sep_voc,sr,['signals/targetEstimate_gfbSOA_Rt', num2str(Rt(jj)),'_beta', num2str(beta(ii)),'.wav']);
        wavsave(dgt_sep_voc,sr,['signals/targetEstimate_DGT_Rt', num2str(Rt(jj)),'.wav']);
        
        % Call the PEASS Toolbox that computes a perceptual score based on PEMO-Q 
        disp('Computing objective scores using PEASS...')
        options.destDir = 'signals/';
        options.segmentationFactor = 1; % increase this integer if you experienced "out of memory" problems
        % Set inputs
        originalFiles = {...
            'signals/femvocd1.wav';...
            'signals/ronin55d1.wav'};
        TarEst_Hann = ['signals/targetEstimate_Hann_Rt', num2str(Rt(jj)),'_beta', num2str(beta(ii)),'.wav'];
        TarEst_gfbSOA = ['signals/targetEstimate_gfbSOA_Rt', num2str(Rt(jj)),'_beta', num2str(beta(ii)),'.wav'];
        TarEst_gfbOPT = ['signals/targetEstimate_gfbOPT_Rt', num2str(Rt(jj)),'_beta', num2str(beta(ii)),'.wav'];
        TarEst_dgt = ['signals/targetEstimate_DGT_Rt', num2str(Rt(jj)),'.wav'];
       
        % Call main function        
        res_gfbSOA = PEASS_ObjectiveMeasure(originalFiles,TarEst_gfbSOA,options);
        res_gfbOPT = PEASS_ObjectiveMeasure(originalFiles,TarEst_gfbOPT,options);
        res_Hann = PEASS_ObjectiveMeasure(originalFiles,TarEst_Hann,options);
        res_dgt = PEASS_ObjectiveMeasure(originalFiles,TarEst_dgt,options);
        
        disp('done')
        
        % Display results
        disp('Signal-to-distortion ratios SDR:')
        fprintf('gfbSOA SDR = %.1f dB\n',res_gfbSOA.SDR);
        fprintf('gfbOPT SDR = %.1f dB\n',res_gfbOPT.SDR);
        fprintf('Hann-Audlet SDR = %.1f dB\n',res_Hann.SDR);
        fprintf('DGT SDR = %.1f dB\n',res_dgt.SDR);
        
        disp('Signal-to-alias ratios (SAR):')
        fprintf('gfbSOA SAR = %.1f dB\n',res_gfbSOA.SAR);
        fprintf('gfbOPT SAR = %.1f dB\n',res_gfbOPT.SAR);
        fprintf('Hann-Audlet SAR =%.1f dB\n',res_Hann.SAR);
        fprintf('DGT SAR = %.1f dB\n',res_dgt.SAR);


        disp('Overall Perceptual Scores:')
        fprintf('gfbSOA OPS = %.f/100\n',res_gfbSOA.OPS);
        fprintf('gfbOPT OPS = %.f/100\n',res_gfbOPT.OPS);
        fprintf('Hann-Audlet OPS = %.f/100\n',res_Hann.OPS);
        fprintf('DGT OPS = %.f/100\n',res_dgt.OPS);
        
        disp('Target-related Perceptual Scores:')
        fprintf('gfbSOA TPS = %.f/100\n',res_gfbSOA.TPS);
        fprintf('gfbOPT TPS = %.f/100\n',res_gfbOPT.TPS);
        fprintf('Hann-Audlet TPS = %.f/100\n',res_Hann.TPS);
        fprintf('DGT TPS = %.f/100\n',res_dgt.TPS);
    end
end

close all

if PLOT % Plot some of the results
    figure(1);
    plotfilterbank(MASK_Hann',a_H,fc,sr,60,'audtick');
    title(['Mask for Hann prototype, $\beta$ = ' num2str(beta(1))], 'interpreter','latex');
%     colormap(1-gray)
%     print -deps maskHann.eps
   
    figure(2);
    plotfilterbank(MASK_GT',a_gt,fc,sr,60,'audtick');
    title(['Mask for compactly-supported gammatone prototype, $\beta$ = ' num2str(beta(1))],'interpreter','latex');
%     colormap(1-gray)
%     print -deps maskGT.eps
    
    figure(3);
    plotdgtreal(MASKGAB,20,sr,60);
    title('Mask for DGT using Hann prototype');
    colormap(1-gray)
    print -deps maskDGT.eps

    figure(4)
    plotfilterbank(c_gfb,a_gt,fc,sr,60,'audtick');
    title('Analysis of source mixture using gammatone FB');
%     colormap(1-gray)
%     print -deps mixtureGFB.eps

    figure(5)
    plotfilterbank(c_hann,a_H,fc,sr,60,'audtick');
    title('Analysis of source mixture using Hann-Audlet');

    figure(6)
    plotdgtreal(c_dgt,a_gab,M,sr,60);
    title('Analysis of source mixture using DGT');
%     colormap(1-gray)
%     print -deps mixtureDGT.eps

    figure(7);
    c_gfb_voc = filterbank(gfbOPT_sep_voc,gfb,a_gt);
    plotfilterbank(c_gfb_voc,a_gt,fc,sr,60,'audtick');
    title(['Voice signal separated by gfbOPT for Rt = ',num2str(Rt(1))]);
%     colormap(1-gray)
%     print -deps TarEstgfbOPT.eps

    figure(8);
    c_gfb_voc2 = filterbank(gfbSOA_sep_voc,gfb,a_gt);
    plotfilterbank(c_gfb_voc2,a_gt,fc,sr,60,'audtick');
    title(['Voice signal separated by gfbSOA for Rt = ',num2str(Rt(1))]);
    
    figure(9);
    c_erb_voc = filterbank(hann_sep_voc,erblets,a_H);
    plotfilterbank(c_erb_voc,a_H,fc,sr,60,'audtick');
    title(['Voice signal separated by Hann-Audlet for Rt = ',num2str(Rt(1))]);
%     colormap(1-gray)
%     print -deps TarEsthannOPT.eps
    
    figure(10);
    c_dgt_voc = dgtreal(dgt_sep_voc,{'hann',M},a_gab,M);
    plotdgtreal(c_dgt_voc,a_gab,M,sr,60);
    title(['Voice signal separated by DGT for Rt = ',num2str(Rt(1))]);
%     colormap(1-gray)
%     print -deps TarEstDGT.eps
end

% eof