% This function is part of the reproducible research addendum of the research
% paper:
% T. Necciari, N. Holighaus, P. Balazs, Z. Prusa, P. Majdak, and O. Derrien
% "Audlet Filter Banks: A Versatile Analysis/Synthesis Framework using Auditory Frequency Scales"
%
% This file reproduces the second experiment (nonuniform FB setting) of the
% gammatone FB's evaluation. It computes the reconstruction error
% (SNR, in dB). The values displayed in the console should be very similar
% to those reported in Table~2 in the manuscript.
%
% To run this file, you will need the LTFAT Toolbox V.2.2.1 or later,
% available at: http://ltfat.github.io
% 
% Authors: Nicki Holighaus & Thibaud Necciari

% Set signal parameters.
Ls = 30000;
sig = randn(Ls,1);
fs = 44100;

% Set FB settings
fmin = 20;
fmax = 20000;
ERBspacing = 1./1.21;
O = [1,2,4,6,8];

red_values = 2*O;
true_red = zeros(6,1);

DF = zeros(5,50);

% Set of optimal d_k's according to Strahl & Mertins, 2009, appendix B.
DF(1,:) = [128*ones(1,10),64*ones(1,23),32*ones(1,16),16];
DF(2,:) = [64*ones(1,8),32*ones(1,28),16*ones(1,12),8*ones(1,2)];
DF(3,:) = [32*ones(1,10),16*ones(1,21),8*ones(1,19)];
DF(4,:) = [16*ones(1,20),8*ones(1,24),4*ones(1,5),2];
DF(5,:) = [16*ones(1,10),8*ones(1,29),4*ones(1,7),2*ones(1,4)];

RE_opt = zeros(5,1);

% Generate gammatone filters
[g,a,~,~]=erbfilters(fs,Ls,'gammatone','spacing',ERBspacing,'fractional','fmin',fmin,'fmax',fmax,'trunc_at',0);
a_orig = a;
DF_orig = a_orig(:,1)./a_orig(:,2);
M = numel(a);

true_red(end) = sum(2*a_orig(:,2)./a_orig(:,1))-sum(a_orig([1,end],2)./a_orig([1,end],1));

for kk = 1:5
    disp(['Currently at FRACOPTTEST, kk=',num2str(kk)]);
    temp_a = [a(1,:);repmat(a(1,1),numel(g)-2,1),ceil(a(1,1)./DF(kk,:)).';a(end,:)];
    scal = [1;DF(kk,:)'./DF_orig(2:end-1);1];
    temp_g = filterbankscale(g,sqrt(scal)');    
    Fg = frame('filterbankreal',temp_g,temp_a,M);
    temp_coef = filterbank(sig,temp_g,temp_a);
    temp_sr = ifilterbankiter(temp_coef,temp_g,temp_a,'pcg');          
    RE_opt(kk) = 20*log10(norm(sig-temp_sr,2)/norm(sig,2));
    a_new = temp_a(:,1)./temp_a(:,2);
    true_red(kk) = sum(2./a_new)-sum(1./a_new([1,end]));
end

disp('\n Optimal set of d_k''s: values of the SNR in dB:\n')
RE_opt
disp('for the following redundancies:\n')
true_red

% Set of d_k's that are inversely proportional to the filters' bandwidth 
[g,a,~,L]=erbfilters(fs,Ls,'gammatone','spacing',ERBspacing,'fractional','fmin',fmin,'fmax',fmax,'trunc_at',0);
a_orig = a;
DF_orig = a_orig(:,1)./a_orig(:,2);
M = numel(a);
RE_prop = zeros(5,1);
org_red = sum(2./DF_orig)-sum(1./DF_orig([1,end]));

true_red_prop = zeros(6,1);
true_red_prop(end) = sum(2*a_orig(:,2)./a_orig(:,1))-sum(a_orig([1,end],2)./a_orig([1,end],1));

for kk = 1:5
    disp(['Currently at FRACPROPTEST, kk=',num2str(kk)]);
    [temp_g,temp_a,~,~]=erbfilters(fs,Ls,'gammatone','spacing',ERBspacing,'fractional','fmin',fmin,'fmax',fmax,'trunc_at',0,'redtar',red_values(kk));
    Fg = frame('filterbankreal',temp_g,temp_a,M);
    temp_coef = filterbank(sig,temp_g,temp_a);
    temp_sr = ifilterbankiter(temp_coef,temp_g,temp_a,'pcg');        
    RE_prop(kk) = 20*log10(norm(sig-temp_sr,2)/norm(sig,2));
    a_new = temp_a(:,1)./temp_a(:,2);
    true_red_prop(kk) = sum(2./a_new)-sum(1./a_new([1,end]));
end

disp('\n Set of d_k''s that are inversely proportional to the filters'' bandwidth: values of the SNR in dB:\n')
RE_prop
disp('for the following redundancies:\n')
true_red_prop