% This function is part of the reproducible research addendum of the research
% paper:
% T. Necciari, N. Holighaus, P. Balazs, Z. Prusa, P. Majdak, and O. Derrien
% "Audlet Filter Banks: A Versatile Analysis/Synthesis Framework using Auditory Frequency Scales"
%
% This file reproduces the first experiment (uniform FB setting) of the
% gammatone FB's evaluation. It consists in a frame bounds analysis and the
% computation of the reconstruction error (SNR, in dB).
% The values displayed in the console should be very similar
% to those reported in Table~1 in the manuscript.
%
% To run this file, you will need the LTFAT Toolbox V.2.2.1 or later,
% available at: http://ltfat.github.io
% 
% Authors: Nicki Holighaus & Thibaud Necciari

% Set signal parameters.
Ls = 30000;
sig = randn(Ls,1);
fs = 44100;

% Set FB settings
fmin = 20;
fmax = 20000;
ERBspacing = 1./[1.2,1.8,2.42,3.65];% Spacing experimentally derived to 
% obtain the same number of filters as in Strahl & Mertins (2009) in the
% frequency range [20; 20000] Hz.
D = [1,2,4,6,8];
FBratio_funiform = zeros(4,4);
RE_funiform = zeros(4,4);
    
for kk = 1:4
    % Generate gammatone filters
    [g,a,fc,L] = audfilters(fs,Ls,'fractional','gammatone','spacing',ERBspacing(kk),'fmin',fmin,'fmax',fmax,'trunc_at',0,'redmul',4);
    a_old = a(:,1)./a(:,2);
    M = numel(a);
    for jj = 1:5
        disp(['Currently at FRACUNITEST, kk=',num2str(kk),', jj=',num2str(jj)]);
        temp_a = [a(1,:);repmat([a(1,1),a(1,1)/D(jj)],numel(g)-2,1);a(end,:)];
        a_new = temp_a(:,1)./temp_a(:,2);
        ratio = a_new./a_old;
        temp_g = filterbankscale(g,sqrt(ratio).'); 
        Fg = frame('filterbankreal',temp_g,temp_a,M);
       if jj == 1
%            For D=1, frame bounds can be computed by analyzing the
%            frequency response of the FB
             temp = filterbankresponse(temp_g,temp_a,Ls,'real');
             FBratio_funiform(kk,jj) = max(temp)/min(temp);
       else
%            For D>1, frame bounds are computed by an eigenvalues analysis
%            of the frame operator.
           FBratio_funiform(kk,jj) = framebounds(Fg,Ls,'iter','tol',10e-5);    
       end     
        temp_coef = filterbank(sig,temp_g,temp_a);
        temp_sr = ifilterbankiter(temp_coef,temp_g,temp_a,'pcg');  
        RE_funiform(kk,jj) = 20*log10(norm(sig-temp_sr,2)/norm(sig,2));
    end
end

disp('\nFrame bound ratio B/A:\n')
FBratio_funiform

disp('\nSNR, in dB:\n')
RE_funiform

% eof